
'
'==================
Class EditBoxObject
'==================

  'TEXT LOCATION (FILE .. CHARACTER)
  '---------------------------------
  sys    ptxt       'pointer to text object
  sys    et         'number of lines

  sys    SelLine    'picker
  sys    SelWord    'picker
  sys    SelChar    'picker

  sys    CurLinePos 'current text line position in file 1..end
  sys    CurWordPos 'current selected word position in text line
  sys    CurCharPos 'current character position in text line (caret to the left of this char)
  string CurWord    'current selected word (highlighted wherever it appears on screen)
  string TextName   'name of text (usually the filename or full pathname)

  'OBJECT
  '------

  sys    ido   'object identity number
  sys    ndis  'display refresh flag (new display)
  sys    oms   'flag for moving and scaling
  sys    lino  'flage to display line numbers
  sys    ofs   'line offset for first line on display
  sys    q     'how many lines to display

  'VIEWPORT (PIXEL) METRICS
  '------------------------

  sys    ewh   'edit window height
  sys    eww   'edit window width
  sys    ewx1  'coord
  sys    ewx2  'coord
  sys    ewy1  'coord
  sys    ewy2  'coord


  'SPATIAL METRICS
  '---------------

  single d     'size of character (font)
  single ypd   'base padding
  single vpr   '
  single spd   'micro-scrolling speed factor
  single yscrl 'scroller line displacement (vertical)
  single xscrl 'scroller line displacement (horizontal)
  single scrc  'caret offset used for horizontal scrolling
  single xd    'displacement of the control from the eyeview centre
  single yd    'displacement of the object from the eyeview centre
  single sclx  'x scale of the object
  single scly  'y scale of the object
  single od    'prior font size
  single oxd   'prior value of xd before a move (mouse drag)
  single oyd   'prior value of yd before a move (mouse drag)
  single osclx 'prior value of sclx before rescaling (mouse drag/pull)
  single oscly 'prior value of scly before rescaling (mouse drag/pull)
  sys onl      'prior value of q line count before rescaling (mouse drag/pull)


  'PICK ZONES
  '----------

  sys    txd[064]   'word/char position on line
  single txc[256]   'word metrics x,y,w,h
  single txo[256]   'char metrics x,y,w,h
  string txa[064]   'words on line

  'RESCALING AND MOVING
  '--------------------

  single pxd,pyd,pxc,pyc
  sys setrsm


  '----------------------------------------------------------------------------------
  function RenderText(s as string, byval x as single, y as single, byval d as single)
  '==================================================================================
  '
  if not len s then exit sub
  glPushAttrib GL_LIST_BIT
  glPushMatrix
  glLoadIdentity
  glListBase 1000
  glTranslatef x, y, -1
  glscalef d, d*.75, 0.01
  glpushMatrix
  glCallLists len(s), GL_UNSIGNED_BYTE, byval *s
  glPopMatrix
  glPopMatrix
  glPopAttrib
  end function


  '----------------------------------------------------------------------------------------------
  function PutBoxArea(byval x as single, byval y as single, byval w as single, byval h as single)
  '==============================================================================================
  '
  glPushMatrix
  glLoadIdentity
  gltranslatef x,y,-1.0
  glbegin GL_QUADS
  'clockwise
  glvertex3f 0, 0, 0
  glvertex3f 0, h, 0
  glvertex3f w, h, 0
  glvertex3f w, 0, 0
  glend
  glpopmatrix
  end function


  '----------------------------------------------------------
  function GetWordArea(s as string, x as single, y as single)
  '==========================================================
  /*
  TYPE POINTFLOAT
    x AS SINGLE
    y AS SINGLE
  END TYPE
  '
  TYPE GLYPHMETRICSFLOAT
    gmfBlackBoxX AS SINGLE
    gmfBlackBoxY AS SINGLE
    gmfptGlyphOrigin AS POINTFLOAT
    gmfCellIncX AS SINGLE
    gmfCellIncY AS SINGLE
  END TYPE
  */

  indexbase 0
  x=0 : y=gmf[77].gmfCellIncY
  sys a,b,e=len s
  for b=1 to e
    a=asc(s,b)
    x+=gmf[a].gmfCellIncX
    'x-=gmf[a].gmfptGlyphOrigin.x
    'x+=gmf[a].gmfBlackBoxX
  next
  end function



  '---------------------
  method RenderControl()
  '=====================
  '
  '
  'transient variables
  '-------------------
  '
  static sys    a,c,e,f,i,j,n,p,gn,mpx,mpy,dpx,dpy,fl
  static single xp,yp,scv,gf,gfy,v,b,alf,xtxt,ytxt,rto,lmx
  static single s1,s2,s3,s4
  static string tx,tx1,tx2
  '
  '
  static double clip_plane1[4],clip_plane2[4],clip_plane3[4],clip_plane4[4]
  '
  static TextArray * txt : @txt=ptxt
  '
  '
  v=0    'line y offset
  gn=0   'name counter
  '
  '
  '
  'initial values
  '-------------
  '
  if q=0 then setup 1,0,0,1,1,.02,1,0
  '
  'adjust params
  '-------------
  '
  if ndis=0 then
    if isnewview then ndis=2
    'if bleft then ndis=2
  end if
  '
  if ndis then
    '
    et=txt.LastLine
    'vpr=0.414        'viewport: y ratio +/- vpr ewh
    'd=vpr*2*scly/q   'scale size to number of lines in the display buffer
    '
    rto=viewport[4]*.5/vpr
    b=1/aspect 'y/x ratio 'aspect is wwidth/wheight
    '
    ewx1=viewport[3]*(1-sclx*b)*.5+viewport[1]+xd*rto
    ewy1=viewport[4]*(1-scly)*.5+viewport[2]+yd*rto
    eww= viewport[3]*sclx*b
    ewh= viewport[4]*scly
    ewx2=ewx1+eww
    ewy2=ewy1+ewh
    '
    if ofs<0 then ofs=0
    if ndis<2 then yscrl=0
    ndis=0
    '
  end if
  '
  mpx=mposx-ewx1 'mouse x position within control (left)
  mpy=mposy-ewy1 'mouse y position withing control (bottom)
  '
  if bleft then
    if (mpx>0)and(mpx<eww) then
      if (mpy>0)and(mpy<ewh) then
        if exclu=0 then
          focus=ido : exclu=ido 'claim focus
        end if
      end if
    end if
  end if
  '
  '--------------------
  'backgound and border
  '====================
  '
  if picklevel=0 then

    s1=-2 : s2=-1 : s3=0.6 : s4=s3*.2
    '
    glpushmatrix
    gltranslatef xd,yd,0
    glscalef sclx,scly,1
    '
    clip_plane1=>1.0,  0.0,  0.0, vpr  'clip left
    clip_plane2=>-1.0, 0.0,  0.0, vpr  'clip right
    clip_plane3=>0.0,  1.0,  0.0, vpr  'clip below
    clip_plane4=>0.0,  -1.0, 0.0, vpr  'clip above
    '
    glClipPlane(GL_CLIP_PLANE1,clip_plane1)
    glClipPlane(GL_CLIP_PLANE2,clip_plane2)     
    glClipPlane(GL_CLIP_PLANE3,clip_plane3)
    glClipPlane(GL_CLIP_PLANE4,clip_plane4)
    '
    glEnable GL_CLIP_PLANE1
    glEnable GL_CLIP_PLANE2
    glEnable GL_CLIP_PLANE3
    glEnable GL_CLIP_PLANE4

    'background
    '----------
    '
    glbegin GL_QUADS
    glcolor4f   s4,  s4,  s3, .7
    glvertex3f -s1, -s1,  s2
    glcolor4f   s4,  s4,  s3, .7
    glvertex3f -s1,  s1,  s2
    glcolor4f   s4,  s4,  s3, .7
    glvertex3f  s1,  s1,  s2
    glcolor4f   s4,  s4,  s3, .7
    glvertex3f  s1, -s1,  s2
    glend
    '
    glcolor4f   s3,   s3,  s3, 1
    '
    glpopmatrix
    '
  end if
  '
  '-------------
  'DISPLAY LINES
  '=============
  '
  fl=1 : lmx=0 : b=0
  '
  '-------------------
  for i=q to 1 step -1
  '===================
    b=v+yscrl
    v+=d
    j=i+ofs 'data ref
    if j=0 then jmp fwd ndispl
    if (b>0)and(b<1) then
      tx= txt.line(j) '+"  " str(CurLinePos) + "  " str(CurWordPos) +"  " str(Curcharpos) "  " CurWord
      'if focus=ido then
      'if len tx then
        '
        'display line
        '------------
        '
        xp=xd-vpr*sclx
        yp=yd+b+ypd-vpr*scly
        'alf=.5-abs(b-vpr) : alf+=alf : alf= sqr alf 'Fading
        alf=1
        '
        b=xp+xscrl
        '
        if lino then
          '
          'line numbers
          '------------
          '
          glColor4f 0.2,0.4,0.6,alf 'line number text color
          '
          tx1=str(j)+"  "
          GetWordArea tx1, gf,gfy
          RenderText tx1,b,yp,d
          gf*=d
          b+=gf          
          '
        end if
        '
        xtxt=b
        '
        glColor4f 0.7,0.7,0.6,alf 'regular text color
        GetWordArea tx,gf,gfy
        gf*=d 'scale
        '
        if picklevel then
        if focus=ido then
          '
          '
          if picklevel=1 then 'LINE IDENTIFICATION
          '----------------------------------
            '
            'use box in place of text
            '
            glLoadname j
            PutBoxArea b,yp,gf+d*8,d 'extra width to pick empty lines
            '
          elseif picklevel=2 'WORD IDENTIFICATION
          '---------------------------------
            '
            'only interested in the identified line
            '
            if SelLine=j then
              '
              CurLinePos=0 : CurWordPos=0
              CurCharPos=txt.length(j)+1 'default
              c=1 : e=len tx
              do
                if c>e then exit do
                if gn=63 then exit do
                gn+=1 : glLoadname gn
                txd[gn]=c 'record position on line
                tx1=getword tx, c
                GetWordarea tx1, gf, gfy
                gf*=d
                PutBoxArea b,yp,gf,d
                '
                'record words on line and locations for syntax handling
                '
                txa[gn]=tx1          'store word
                txc[gn*4]=>b,yp,gf,d 'store area box
                b+=gf
                '
              end do
              '
            end if
            '
          else 'if picklevel=3 'CHAR IDENTIFICATION
          '-----------------------------------
            '
            if SelLine=j then
              tx1=txa[SelWord] : a=SelWord*4
              b=txc[SelWord*4] 'word offset
              c=1 : e=len tx1
              if e>63 then e=63
              do
                if c>e then exit do
                gn+=1 : glLoadname gn
                tx2=mid tx1,c,1
                GetWordarea tx2, gf, gfy
                gf*=d
                PutBoxArea b,yp,gf,d
                '
                'record character box area on line
                '
                txo[gn*4]=>b,yp,gf,d 'store area box
                b+=gf : c+=1
              end do
              if ldblclk=2 then CurWord=ltrim tx1
              CurWordPos=txd[SelWord]
              CurLinePos=j
            end if
          end if
          '
        end if 'focus=ido
        end if 'picklevel
        '
        '
        if not picklevel then
          '
          'syntax highlighting
          '-------------------
          '
          'comment detection
          '
          c=instr(tx,"'")
          e=instr(tx,";")
          if e then
            if c then
              if c>e then c=e
            else
              c=e
            end if
          end if
          '
          '
          'highlight selected word
          '
          if len CurWord then
            tx2=CurWord
            e=len tx2
            do
              a=instr(tx,tx2)
              if not a then exit do
              if c then
                if a>c then exit do 'do not apply to Basic comments
                c-=e
              end if
              tx1=left(tx,a-1)
              tx=mid(tx,a+e) : a=1
              GetWordArea(tx1,gf,gfy)
              if len tx1 then RenderText tx1,b,yp,d
              b+=(gf*d)
              GetWordArea(tx2,gf,gfy)
              gf*=d
              if gf>0 then
                glColor4f 0.0,0.7,0.7,.5 'background text color
                PutBoxArea b,yp,gf,d
                glColor4f 0.9,0.9,0.1,alf*.5 'highlight
                RenderText tx2,b,yp,d
              end if
              b+=gf
              glColor4f 0.6,0.7,0.6,alf 'standard
            end do
          end if
          '
          '
          'for Basic comments
          '
          if c>0 then a=instr(tx,"'") else a=0
          '
          if a then
            tx1=left(tx,a-1)
            tx=mid(tx,a)
            GetWordArea(tx1,gf,gfy)
            if len tx1 then RenderText tx1,b,yp,d
            b+=(gf*d)
            glColor4f 0.9,0.4,0.1,alf         
          end if
          '
          'main textline
          '
          if len tx then
            GetWordArea(tx1,gf,gfy)
            RenderText tx,b,yp,d
            b+=(gf*d)
          end if
          '
          if lmx<b then lmx=b
          '
          'HIGHLIGHT WORD AND CHARACTER
          '
          if CurLinePos=j then
            if CurWordPos then
              '
              'box matching word area
              '
              tx1=txa[SelWord] : a=SelWord*4
              '
              glColor4f 0.9,0.9,0.1,0.2 'background text color
              PutBoxArea txc[a],txc[a+1],txc[a+2],txc[a+3]
              glColor4f 0.9,0.9,0.1,alf 'highlight text color
              'glColor4f 0,0,0,1 'highlight text color
              RenderText tx1,txc[a],txc[a+1],txc[a+3]
              '
              'highlight character
              '
              a=SelChar*4
              glColor4f 0.9,0.9,0.7,0.2 'highlight char color
              PutBoxArea txo[a],txo[a+1],txo[a+2],txo[a+3]                
              '
            end if
          end if
          '
          if CurLinePos=0 then CurLinePos=1
          '
          'CARET PLACEMENT
          '---------------
          '
          if CurLinePos=j then
            ytxt=yp
            b=xtxt
            if CurCharPos=0 then CurCharPos=txt.length(j)+1
            tx= txt.line(CurLinePos)
            tx1=left tx,CurCharPos-1
            GetWordArea tx1,gf,gfy
            gf*=d
            b+=gf
            glColor4f 0.9,0.9,0.1,alf 'caret color
            PutBoxArea b,yp, d*.125, d 'caret as rectangle
            scrc=b+vpr
          end if 'caret
        end if
      'end if
    end if
    '
    '
    ndispl:
    '
  '-----
  next
  '=====
  '
  'BORDER
  '------
  '
  if picklevel=0 then
    glDisable GL_CLIP_PLANE1
    glDisable GL_CLIP_PLANE2
    glDisable GL_CLIP_PLANE3
    glDisable GL_CLIP_PLANE4
    '
    glpushmatrix
    gltranslatef xd,yd,0
    glscalef sclx,scly,1
    s1=-vpr
    s2=-vpr
    s3=s1+vpr*2
    s4=s2+vpr*2
    '
    glbegin GL_LINE_STRIP
    glvertex3f s1, s2, -1
    glvertex3f s3, s2, -1
    glvertex3f s3, s4, -1
    glvertex3f s1, s4, -1
    glvertex3f s1, s2, -1
    glend
    glpopmatrix
  end if
  '
  '
  '----------------------------
  'SCALING AND MOVING
  '============================
  '
  '
  if bleft then
    '
    acte=1
    '
    '
    if focus<>ido then
      jmp fwd nscale
    end if
    '
    '
    if mpy>0.9*ewh then
      if mpy<ewh then
        '
        'ANCHOR ORIGINAL VALUES
        '
        if oms=0 then
          '
          onl=q : oxd=xd : oyd=yd : od=d
          '
          'LINE COUNT
          '
          if mpx>0.8*eww then
            oms=3
          '
          'MOVING
          '
          elseif mpx>0.4*eww then
            if mpx<0.8*eww then
              oms=4
           end if
          end if
        end if
      end if
    elseif mpy<.1*eww then
      '
      if oms=0 then osclx=sclx : oscly=scly : onl=q : oxd=xd : oyd=yd
      '
      'SCALING
      '
      if mpx>0.8*eww then
        oms=1
      elseif mpx>0.4*eww then
        if mpx<0.8*eww then
          oms=2
        end if
      end if
    end if
    '
    '--------
    dynScale: 'DYNAMIC SCALING AND MOVING WITH MOUSE
    '--------
    '
    if oms then
      '
      dpx=mposx-sposx
      dpy=mposy-sposy
      single a
      '
      if oms=1 then
        '
        sclx=osclx+dpx/(rto*vpr*2) 'X SCALING
        if sclx<.02 then sclx=.02
        xd=oxd+vpr*(sclx-osclx) 'anchor adjustment
        '
        scly=oscly-dpy/(rto*vpr*2) 'Y SCALING
        if scly<.02 then scly=.02
        yd=oyd+vpr*(oscly-scly) 'anchor adjustment
        '
        q=scly*vpr*2/d 'onl*scly/oscly 'line count
        a=scly*vpr*2
        q=a/d   'line count (int truncation)
        ypd=a-q*d 'baseline padding
        ndis=2
        exit method
        '
      elseif oms=2 then
        '
        scly=oscly-dpy/(rto*vpr*2) 'Y SCALING
        if scly<.02 then scly=.02
        yd=oyd+vpr*(oscly-scly)
        a=scly*vpr*2
        q=a/d   'line count
        ypd=a-q*d 'base line padding
        ndis=2
        exit method
        '
      elseif oms=3 then 'LINE COUNT
        '
        d=od+dpy*.0001
        if d<.004 then d=.004
        a=scly*vpr*2
        q=a/d   'line count
        ypd=a-q*d 'base line padding
        'q=scly*vpr*2/d 'onl*scly/oscly     'line count
        ndis=1
        exit method
        '
      elseif oms=4 then
        '
        xd=oxd+dpx/rto 'MOVE X
        yd=oyd+dpy/rto 'MOVE Y
        ndis=2
        exit method
        '
      end if
      '
    end if
    '
  else 'LEFT BUTTON IS UP
    oms=0
  end if
  '
  '------
  nscale:
  '------
  '
  '
  if picklevel then
    glLoadName 0 'fence
    exit method 'no other action when picking
  end if
  '
  if focus<>ido then exit method
  '
  '--------
  'KEYBOARD
  '========
  '
  '
  if keyt then
  if viewmode=5 then
    '
    c=CurLinePos
    a=txt.length(c)+1
    e=GetAsyncKeyState(0x11) and 0x8001 'Check for Ctrl-key
    if e then
      if keyt=34 then 'CTRL-PGDOWN
        CurLinePos=et
        ofs=et-q*.5
        if ofs<0 then ofs=0 
      elseif keyt=33 then 'CTRL-PGUP
        ofs=0 : CurLinePos=1
      end if
      jmp fwd nKeySel
    end if
    '
    select keyt
    case 38  : CurLinePos-=1     'ARROW UP
    case 40  : CurLinePos+=1     'ARROW DOWN
    case 37  : CurCharPos-=1     'ARROW LEFT
    case 39  : CurCharPos+=1     'ARROW RIGHT
    case 36  : CurCharPos=1      'HOME
    case 35  : CurCharPos=a      'END
    case 34  : CurLinePos+=q-2   'PG DOWN
    case 33  : CurLinePos-=(q-2) 'PG UP
    case 46  : keyc=1025         'DELETE
    case 114 'F3
      if len(Curword) then
        e=CurLinePos
        f=CurCharPos
        txt.locate CurWord, e, f
        if e then
          CurlinePos=e : CurCharPos=f+len CurWord
          ofs=e-10
          if ofs<1 then ofs=0
          ndis=1
          keyt=0 : acte=1
          exit method
        end if
      end if
    end select
    '
    'Characters to insert (or delete)
    '--------------------------------
    '
    if keyc then
      tx=txt.line(CurLinePos)
      '
      if keyc=1025 then  'DELETE
        if CurCharPos>txt.length(CurLinePos) then
          tx1=txt.line(CurLinePos)+txt.line(CurLinePos+1)
          txt.delete(CurLinePos+1)
          txt.line(CurLinePos)=tx1
          jmp fwd nkeysel
        else
          CurCharPos+=1
          keyc=8
        end if
      end if
      '
      if keyc=8 then 'BACKSPACE DELETION
        '
        if CurCharPos=1 then
          if CurLinePos>1 then
            CurLinePos-=1
            CurCharPos=txt.length(CurLinePos)+1
            tx2=txt.line(CurLinePos)+txt.line(CurLinePos+1)
            txt.delete(CurLinePos+1)
            txt.line(CurLinePos)=tx2
          end if
        else 'if CurCharPos>1 then
          CurCharPos-=1
          txt.line(CurLinePos)=left(tx,CurCharPos-1)+mid(tx,CurCharPos+1)
        end if
        '
        '
      elseif keyc=13 then 'ENTER
        '
        tx1=txt.line(CurLinePos)
        txt.line(CurLinePos)=left tx1,CurCharPos-1
        CurLinePos+=1
        tx2=mid tx1,CurCharPos
        txt.insert(CurLinePos)=tx2
        CurCharPos=1
        '
      else 'CHAR INSERT
        '
        if txt.length(CurLinePos)=0 then
          tx2=chr keyc
        else
          tx2=left(tx,CurCharPos-1)+chr(keyc)+mid(tx,CurCharPos)
        end if
        '
        txt.line(CurLinePos)=tx2
        '
        CurCharPos+=1 : a+=1
        '
      end if
    end if
    '
    '=======
    nkeySel:
    '=======
    '
    keyc=0 'clear char
    keyt=0 'clear keydown transient
    acte=1 'refresh screen
    '
    'limit checks
    '-----------
    '
    et=txt.LastLine
    '
    if CurCharPos<1 then
      if CurLinePos>1 then
        CurLinePos-=1
        CurCharPos=txt.length(CurLinePos)+1
      else
        CurCharPos=1
      end if
    elseif CurCharPos>a then
      if CurLinePos<et then
        CurCharPos=1
        CurLinePos+=1
      else
        CurCharPos=a
      end if
    end if
    '
    if CurLinePos>et then CurLinePos=et 'could be 0
    if CurLinePos<1 then CurLinePos=1   'never less than 1
    '
    '
    'need screen buffer refresh for whole line scrolling?
    '----------------------------------------------------
    '
    e=0 : a=CurLinePos-c
    '
    if a<0 then
      if curlinepos<ofs+4 then e=-1
    elseif a then
      if curlinepos>ofs+q-2 then e=1
    end if
    '
    if e<0 then
      ofs+=a : ndis=1 'GO UP
      exit method
    elseif e>0 then 'GODOWN
      ofs+=a : ndis=1
      if (ofs+a+q-5)>et then ofs=et-q+5
      if ofs<0 then ofs=0
      exit method
    end if
    '
  end if 'Viewmode 5
  end if 'Keyboard

  '
  '-------------------------
  autoscroll: 'AUTOSCROLLING
  '=========================
  '
  '
  'AUTOSCROLL ZONE CONFINEMENT
  '---------------------------
  '
  if mpx<=0  then jmp fwd nvscroll
  if mpx>eww then jmp fwd nvscroll
  if mpy<=0  then jmp fwd nvscroll
  if mpy>ewh then jmp fwd nvscroll
  '
  if mpx>0.1*eww then jmp fwd nvscroll
  '
  'AUTOSCROLLING DOWN AND UP
  '-------------------------
  '
  if mpy<0.1*ewh then 'SCROLL DOWN
    if (ofs+q-5)<=et then
      acte=1
      scv=d*spd*(10*ewh-mpy*100)*agr/ewh
      yscrl+=scv
      if yscrl>=d then
        n=yscrl/d 'quantise
        ndis=2
        for p=1 to n
          if ofs+q-5<=et then
            ofs+=1 : CurLinePos+=1
          end if
          if yscrl>=d then yscrl-=d
        next
      end if
    end if
    jmp fwd nhscroll
  elseif mpy>0.9*ewh then 'SCROLL UP
    scv=d*spd*(mpy*100-ewh*90)*agr/ewh
    if ofs<=0 then
      yscrl=0
    else
      acte=1
      yscrl-=scv
      if yscrl<0 then
        n=-yscrl/d
        ndis=2
        for p=1 to n
          if ofs>0 then
            ofs-=1 : CurLinePos-=1
          end if
          if yscrl<0 then yscrl+=d
        next
      end if
    end if
    jmp fwd nhscroll
  end if
  '
  '--------
  nvscroll:
  '--------
  '
  '---------------------------------------
  'HORIZONTAL AUTOSCROLLING RIGHT AND LEFT
  '=======================================
  '
  a=0
  if scrc then e=scrc*rto-ewx1 else e=eww*.5
  '
  if mpy>0.2*ewh
    if mpy<0.8*ewh
      'a=mpx
    end if
  end if

  if e>eww*.9 then
    a=e
  elseif e<eww*.1 then
    a=e
  end if
  '
  if a then
    if a>eww*0.9 then xscrl-=spd*(a-eww*.9)/300 : acte=1  'scroll right
    if a<eww*0.1 then xscrl-=spd*(a-eww*.1)/300 : acte=1  'scroll left
    if xscrl>0 then xscrl=0 : acte=0                          'left (margin) scrolling limit
    'lmx=2*vpr*sclx
    'lmx+=(vpr*sclx+d-eww/rto)
    'if lmx<0 then lmx=0
    'if xscrl< -lmx then xscrl=-lmx : acte=0                   'right scrolling limit
    '
  end if
  '
  '--------
  nhscroll:
  '--------
  '
end method


'---------------
method render()
'===============
  '
  if picklevel then RenderControl() : exit method
  '
  SetProjection(0)
  '
  glEnable GL_LINE_SMOOTH
  glDisable GL_LIGHTING
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) 'Enable Alpha Blending
  glEnable(GL_BLEND)                                'Enable Blending
  glAlphaFunc(GL_GREATER,0.1)                       'Set Alpha Testing
  glEnable(GL_ALPHA_TEST)                           'Enable Alpha Testing
  Lighting() 'also disables wireframe mode
  '
  RenderControl()
  '
  glDisable(GL_ALPHA_TEST)
  glDisable GL_BLEND
  glDisable GL_ALPHA_TEST
  '
end method


'-------------------------
method LineNumber() as sys
'=========================
  return CurLinePos
end method


'------------------------------
method GetLine(sys n) as string
'==============================
  if n=0 then n=CurLinePos
  if n then
    TextArray *txt : @txt=ptxt
    return txt.line[n]
  end if
end method


'------------------------------
method PutLine(string s, sys n)
'==============================
  if n=0 then n=CurLinePos
  if n then
    TextArray *txt : @txt=ptxt
    txt.line(n)=s
  end if
end method


'--------------------
method Load(string s)
'====================
  if s
    if TextName<>s
      TextArray *txt : @txt=ptxt
      txt.clear
      txt.load s
      CurLinePos=1
      CurCharPos=1
      xscrl=0
      yscrl=0
      ofs=0
      TextName=s
    end if
  end if
end method


'--------------
method picker()
'==============
  SelLine=0 : SelWord=0 : SelChar=0
  if bleft then
    picklevel=1
    PickPrep : Render : PickResult
    if SelObject then
      SelLine=SelObject : picklevel=2
      PickPrep : Render : PickResult
      if SelObject then
        SelWord=SelObject : picklevel=3
        PickPrep : Render : PickResult
        SelChar=SelObject
        if selchar then
          CurCharPos=CurWordPos+SelChar
        end if
      end if
    end if
  end if
  picklevel=0
end method


'----------------------------------------------------------
method setup(sys id, single ox, oy, sx, sy, sl, sys ln, pt)
'==========================================================
  '
  'STANDARD SETTINGS
  '
  ido=id      'identity number
  sclx=sx     'standard x scale
  scly=sy     'standard x scale
  xd=ox       'displacement from center
  yd=oy       'displacement from center
  ofs=0       'text line offset
  q=sy/sl     'number of lines in display buffer
  spd=.0625   'scrolling speed factor
  lino=ln     'line numbers
  '
  Curword="" : tx="" : tx1="" : tx2="" 'temp strings
  '
  ndis=1 'flag new display
  '
  vpr=0.414        'viewport: y ratio +/- vpr ewh
  'd=vpr*2*scly/q   'line scale size to number of lines in the display buffer
  d=sl*vpr*2       'line size
  ypd=0            'base padding
  '
  ptxt=pt
  '
end method


'------------------------------------------------------------------
method RescaleMove(single nxd, single nyd , single nxc, single nyc)
'==================================================================
  if setrsm then
    xd+=nxd-pxd
    yd+=nyd-pyd
    sclx*=nxc/pxc
    scly*=nyc/pyc
  end if
  pxd=nxd : pyd=nyd : pxc=nxc : pyc=nyc : setrsm=1
end method


'-----------
method enq()
'===========
  print recordof this
end method


'=======================
end class 'EditBoxObject
'=======================


