'ConsoleC class file
'Last modified:
'2011.07.08 05:01 KS

include once "BaseUtils.inc"

type SMALL_RECT
	SHORT Left
	SHORT Top
	SHORT Right
	SHORT Bottom
end type


type COORD
	SHORT X
	SHORT Y
end type


type CONSOLE_SCREEN_BUFFER_INFO
	COORD	   dwSize
	COORD	   dwCursorPosition
	WORD	   wAttributes
	SMALL_RECT srWindow
	COORD	   dwMaximumWindowSize
end type


extern lib "kernel32.dll"
    sys AllocConsole()
    sys FreeConsole()
    sys FillConsoleOutputCharacterA( sys hConsoleOutput, char cCharacter, sys nLength, COORD dwWriteCoord, sys *lpNumberOfCharsWritten )
    sys GetCommandLineA()
    sys GetStdHandle( sys handle )
    sys WriteConsoleA( sys hConsoleOutput, zstring *lpBuffer, sys nNumberOfCharsToWrite, *lpNumberOfCharsWritten, void *lpReserved )
    sys ReadConsoleA ( sys hConsoleInput, zstring *lpBuffer, sys nNumberOfCharsToRead, *lpNumberOfCharsRead, void *pInputControl )
    sys SetConsoleTitleA( Asciiz *lpConsoleTitle )
    sys SetConsoleMode( sys handle,  dwMode )
    sys GetConsoleMode( sys handle, *lpMode )
    sys GetConsoleScreenBufferInfo( sys hConsoleOutput, CONSOLE_SCREEN_BUFFER_INFO *lpConsoleScreenBufferInfo )
    sys GetConsoleTitleA( Asciiz *lpConsoleTitle, sys nSize )
    sys SetConsoleCursorPosition( sys hConsoleOutput, COORD dwCursorPosition )
    sys SetConsoleTextAttribute( sys hConsoleOutput, sys wAttributes )
    sys SetConsoleWindowInfo( sys hConsoleOutput, sys bAbsolute, SMALL_RECT *lpConsoleWindow )
end extern

'standard handles
#define STD_INPUT_HANDLE                -10
#define STD_OUTPUT_HANDLE               -11
#define STD_ERROR_HANDLE                -12 
'Text Attributes
#define FOREGROUND_BLUE	                1
#define FOREGROUND_GREEN	            2
#define FOREGROUND_RED	                4
#define FOREGROUND_INTENSITY	        8
#define BACKGROUND_BLUE	                16
#define BACKGROUND_GREEN	            32
#define BACKGROUND_RED	                64
#define BACKGROUND_INTENSITY	        128
'I made these up
'no fore color specified means black fore color
#define DEFAULT_COLOR                   1 or 2 or 4
#define F_DARK_GREEN                    2
#define F_GREEN                         2 or 8
#define F_DARK_BLUE                     1
#define F_BLUE                          1 or 8
#define F_DARK_RED                      4
#define F_RED                           4 or 8
#define F_DARK_YELLOW                   2 or 4
#define F_YELLOW                        2 or 4 or 8
#define F_DARK_PURPLE                   1 or 4
#define F_PURPLE                        1 or 4 or 8
#define F_DARK_CYAN                     2 or 1
#define F_CYAN                          2 or 1 or 8
#define F_DARK_GREY                     8
#define F_GREY                          1 or 2 or 4
#define F_WHITE                         1 or 2 or 4 or 8
'no back color specified means black back color
#define B_DARK_GREEN                    32
#define B_GREEN                         32 or 128
#define B_DARK_BLUE                     16
#define B_BLUE                          16 or 128
#define B_DARK_RED                      64
#define B_RED                           64 or 128
#define B_DARK_YELLOW                   32 or 64
#define B_YELLOW                        32 or 64 or 128
#define B_DARK_PURPLE                   16 or 64
#define B_PURPLE                        16 or 64 or 128
#define B_DARK_CYAN                     32 or 16
#define B_CYAN                          32 or 16 or 128
#define B_DARK_GREY                     128
#define B_GREY                          16 or 32 or 64
#define B_WHITE                         16 or 32 or 64 or 128
'end Text Attributes
#if (_WIN32_WINNT >= 0x0501)
    #define CONSOLE_FULLSCREEN_MODE	1
    #define CONSOLE_WINDOWED_MODE	2
#endif
#define CTRL_C_EVENT                    0
#define CTRL_BREAK_EVENT                1
#define CTRL_CLOSE_EVENT                2
#define CTRL_LOGOFF_EVENT               5
#define CTRL_SHUTDOWN_EVENT             6
#define ENABLE_LINE_INPUT               2
#define ENABLE_ECHO_INPUT               4
#define ENABLE_PROCESSED_INPUT          1
#define ENABLE_WINDOW_INPUT             8
#define ENABLE_MOUSE_INPUT              16
#define ENABLE_INSERT_MODE              32
#define ENABLE_QUICK_EDIT_MODE          64
#define ENABLE_EXTENDED_FLAGS           128
#define ENABLE_AUTO_POSITION            256
#define ENABLE_PROCESSED_OUTPUT         1
#define ENABLE_WRAP_AT_EOL_OUTPUT       2
#define KEY_EVENT                       1
#define MOUSE_EVENT                     2
#define WINDOW_BUFFER_SIZE_EVENT        4
#define MENU_EVENT                      8
#define FOCUS_EVENT                     16
#define CAPSLOCK_ON                     128
#define ENHANCED_KEY                    256
#define RIGHT_ALT_PRESSED               1
#define LEFT_ALT_PRESSED                2
#define RIGHT_CTRL_PRESSED              4
#define LEFT_CTRL_PRESSED               8
#define SHIFT_PRESSED                   16
#define NUMLOCK_ON                      32
#define SCROLLLOCK_ON                   64
#define FROM_LEFT_1ST_BUTTON_PRESSED    1
#define RIGHTMOST_BUTTON_PRESSED        2
#define FROM_LEFT_2ND_BUTTON_PRESSED    4
#define FROM_LEFT_3RD_BUTTON_PRESSED    8
#define FROM_LEFT_4TH_BUTTON_PRESSED    16
#define MOUSE_MOVED	                    1
#define DOUBLE_CLICK	                2
#define MOUSE_WHEELED	                4

class ConsoleC
    private
        dim CONSOLE_SCREEN_BUFFER_INFO mSBI
        sys mhIn
        sys mhOut
        sys mhErr
    public
        method Constructor( optional string aTitle = "Oxygen Console" )
            AllocConsole
            SetTitle aTitle
            mhIn  = GetInputHandle
            mhOut = GetOutputHandle
            mhErr = GetErrorHandle
            GetSBI mSBI
        end method
        
        method Destructor()
            FreeConsole
        end method
        
        method GetArgs() as string
            zstring* args  
            @args = GetCommandLineA
            return args
        end method        
        
        method Print( string aString )
            sys aStringLen, charsWritten
            aStringLen = len aString
            WriteConsoleA mhOut, aString, aStringLen, charsWritten, byval 0
        end method

        method Input( string aString ) as string
            sys charsRead      
            zstring buffer[255]      
            this.Print aString 'without this, it uses oxygen's default print
            ReadConsoleA mhIn, buffer, 254, charsRead, byval 0
            return left$ buffer, charsRead
        end method
        
        method Fill( char aChar, sys aNumToFill, sys aX, sys aY ) as sys
            sys numCharsWritten
            COORD crd => ( aX, aY )
            FillConsoleOutputCharacterA( mhOut, aChar, aNumToFill, crd, numCharsWritten ) 
            return numCharsWritten
        end method

        method GetKey() as sys
            sys charsRead, mode
            zstring buffer[2]
            mode = GetMode mhIn
                SetMode mhIn, 0 'disables echo to screen
                ReadConsoleA mhIn, buffer, 1, charsRead, byval 0
            SetMode mhIn, mode 
            return ?buffer
        end method
        
        method GetSBWidth() as sys
            GetSBI mSBI
            return mSBI.dwSize.X
        end method
        
        method GetSBHeight() as sys
            GetSBI mSBI
            return mSBI.dwSize.Y
        end method

        method GetCX() as sys
            GetSBI mSBI
            return mSBI.dwCursorPosition.X
        end method
        
        method GetCY() as sys
            GetSBI mSBI
            return mSBI.dwCursorPosition.Y
        end method
        
        method GetMaxWidth() as sys
            GetSBI mSBI
            return mSBI.dwMaximumWindowSize.X
        end method
        
        method GetMaxHeight() as sys
            GetSBI mSBI
            return mSBI.dwMaximumWindowSize.Y
        end method
        
        method GetLeft() as sys
            GetSBI mSBI
            return mSBI.srWindow.Left
        end method
        
        method GetTop() as sys
            GetSBI mSBI
            return mSBI.srWindow.Top
        end method
        
        method GetRight() as sys
            GetSBI mSBI
            return mSBI.srWindow.Right
        end method
        
        method GetBottom() as sys
            GetSBI mSBI
            return mSBI.srWindow.Bottom
        end method
                                
        method GetTitle() as string
            asciiz title[255]
            sys titleLen
            titleLen = GetConsoleTitleA title, 254
            if titleLen then
                return left$ title, titleLen
            else 
                return "Error in getting console title :("
            end if
        end method
        
        method SetTitle( string aString )
            SetConsoleTitleA aString
        end method
        
        method SetRect( short aLeft, aTop, aRight, aBottom ) as sys
            SMALL_RECT aRect => ( aLeft, aTop, aRight, aBottom )
            return SetConsoleWindowInfo mhOut, 1, aRect
        end method
        
        method SetCPos( sys aX, sys aY ) as sys
            COORD crd => ( aX, aY )
            return SetConsoleCursorPosition mhOut, crd
        end method
        
        method SetColor( sys aAttribute ) as sys
            return SetConsoleTextAttribute mhOut, aAttribute
        end method
        
        method Skip( optional sys aNumCRLF = 1 )
            sys x
            for x = 1 to aNumCRLF
                this.Print crlf$
            next
        end method
        
        method WaitKey( optional sys aNumCRLF = 1 )
            sys x
            for x = 1 to aNumCRLF
                this.Print crlf$
            next
            this.Print "Press any key to continue."
            GetKey
        end method
        
    private
        method GetSBI( CONSOLE_SCREEN_BUFFER_INFO *aSBI ) as sys
            return GetConsoleScreenBufferInfo mhOut, aSBI 
        end method
        
        method GetMode( sys aStdHandle ) as sys
            sys mode
            GetConsoleMode aStdHandle, mode
            return mode
        end method
        
        method SetMode( sys aStdHandle, sys aMode ) as sys
            return SetConsoleMode aStdHandle, aMode
        end method
        
        method GetInputHandle() as sys
            return GetStdHandle STD_INPUT_HANDLE
        end method
        
        method GetOutputHandle() as sys
            return GetStdHandle STD_OUTPUT_HANDLE
        end method
        
        method GetErrorHandle() as sys
            return GetStdHandle STD_ERROR_HANDLE
        end method
end class

'Notes:
'a very nice console tutorial website: 
'http://www.adrianxw.dk/SoftwareSite/index.html

'MSDN: http://msdn.microsoft.com/en-us/library/ms682087(v=VS.85).aspx