#If 0
  =============================================================================
   Program NAME:
   Author      :
   Date        :
   Version     :
   Description :
  =============================================================================
  'COPYRIGHT AND PERMISSION NOTICE
  '============================================================================

  =============================================================================
#ENDIF

  #COMPILE DLL
  #REGISTER NONE
  #DIM ALL


  #Resource "thinBasic_CString.PBR"

  #INCLUDE ONCE "WIN32API.INC"
  #INCLUDE "..\thinCore.INC"


  '----------------------------------------------------------------------------
  ' Class CString implements a module class to be used for teaching purposes
  ' It shows how to create a class, how to add modules, how to instruct 
  ' thinBasic about class and module existence at runtime
  '----------------------------------------------------------------------------
  Class CString                 

    '----------------------------------------------------------------------------
    '---Class Private variables
    '----------------------------------------------------------------------------
    Instance lString   As String


    '----------------------------------------------------------------------------
    '---Define here any Class Method (if needed)
    '---Class methods are visible only from inside the class
    '---so they can be called only from inside other methods.
    '----------------------------------------------------------------------------
    'Class Method MyClassMethod() As Long
    'End Method

    '----------------------------------------------------------------------------
    ' We need to define an interface that will contain public methods
    '----------------------------------------------------------------------------
    Interface icString

      Inherit IUnknown

      '----------------------------------------------------------------------------
      ' Assign a new string to class string
      '----------------------------------------------------------------------------
      Method sSet(Optional ByVal NewString As String) As String
        
        '---First of all we will return old class instance value
        Method = lString                         
        
        '---Than we assign new value to class instance
        lString = NewString
      End Method

      '----------------------------------------------------------------------------
      ' Get string from class instance variable
      '----------------------------------------------------------------------------
      Method sGet() As String
        
        '---return class instance value
        Method = lString                         

      End Method

      '----------------------------------------------------------------------------
      ' Get left part of class instance variable
      '----------------------------------------------------------------------------
      Method sLeft(ByVal nChars As Long) As String
        
        '---return left part of class instance variable
        Method = Left$(lString, nChars)

      End Method

      '----------------------------------------------------------------------------
      ' Get right part of class instance variable
      '----------------------------------------------------------------------------
      Method sRight(ByVal nChars As Long) As String
        
        '---return right part of class instance variable
        Method = Right$(lString, nChars)

      End Method

      '----------------------------------------------------------------------------
      ' Get mid part of class instance variable
      '----------------------------------------------------------------------------
      Method sMid(ByVal lStart As Long, ByVal nChars As Long) As String
        
        '---return mid part of class instance variable
        Method = Mid$(lString, lStart, nChars)

      End Method

      '----------------------------------------------------------------------------
      ' Get string lenght
      '----------------------------------------------------------------------------
      Method sLen() As Long
        
        '---return lenght of instance value
        Method = Len(lString)

      End Method

    End Interface

  End Class
  '----------------------------------------------------------------------------
  '----------------------------------------------------------------------------



  '--------------------------------------------------------------------------------------
  '---WRAPPERS to simulate an object using a PTR
  '--------------------------------------------------------------------------------------
  ' Here we are. In order to let thinBasic know about a class, we need to wrap
  ' some, or all of the class interface methods defined above.
  ' This will also let us choose any syntax of our method  
  '--------------------------------------------------------------------------------------

  '--------------------------------------------------------------------------------------
  ' We need to create a constructor wrapper function
  ' Constructor function will be executed by thinBasic when the class will be instantiated
  ' in script using the NEW keyword
  '--------------------------------------------------------------------------------------
  ' IMPORTANT:
  '   ALL wrapper functions MUST HAVE a "BYVAL pClass AS LONG" parameter.
  '   pClass will contain a pointer to the class
  '   This pointer is usually defined in the constructor (this function) using the CLASS
  '   keyword
  '--------------------------------------------------------------------------------------
  Function CString_Create(  _
                        ByVal pClass As Long _
                        ) As Ext
  ' We want Syntax of constructor to be something like: 
  '   Class = New cString[(InitialStringData)]
  ' Example
  '   MyCString = New cString("Initial value")
  '--------------------------------------------------------------------------------------
    Local sBuffer As String
    Local pp      As Byte

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---Optionally checks if there is a (
    pp = thinBasic_CheckOpenParens_Optional
    If pp Then
      '---If yes, then parse string expression and )
      thinBasic_ParseString sBuffer
      thinBasic_CheckCloseParens_Mandatory
    End If

    '---If all went fine ...
    If thinBasic_ErrorFree Then

      'Here is the magic !!!!!!!!!!!!
      'Use the magic REDIM ... AT and the trick is done
      'We are defining a new class interface variable at the pointer location allocated by thinBasic and passed to all methods
      ReDim iClass(1& To 1&) As icString At pClass

      '---We instantiate a new class
      iClass(1&) = Class "cString"   
      '---We call sSet in order to assign initial value
      iClass(1&).sSet(sBuffer)

    'else
      '---emit some error
    End If

  End Function


  '--------------------------------------------------------------------------------------
  Function CString_Free( _
                        ByVal pClass As Long _
                      ) As Long
  '--------------------------------------------------------------------------------------
    
    '---Well, there is nothing we need to do when script class is released
    '---but just in case you allocated some dynamic memory, you will be responsible to
    '---release it. And the best place is the destructor
    If pClass Then
      '---Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass

      iClass(1&).sSet("")
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Get(  _
                        ByVal pClass As Long _
                        ) As String
  ' Syntax: 
  '   cString.Get
  '--------------------------------------------------------------------------------------

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sGet()

    'else
      '---emit some error
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Set(  _
                        ByVal pClass As Long _
                        ) As String
  ' Syntax: 
  '   cString.Set(SomeStringData)
  '--------------------------------------------------------------------------------------
    Local sBuffer As String
    Local pp      As Byte

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    pp = thinBasic_CheckOpenParens_Optional
    '---parse string expression
    thinBasic_ParseString sBuffer
    If pp Then thinBasic_CheckCloseParens_Mandatory

    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sGet()
      
      '---We call sSet in order to assign value
      iClass(1&).sSet(sBuffer)
    'else
      '---emit some error
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Left(  _
                        ByVal pClass As Long _
                        ) As String
  ' Syntax: 
  '   cString.Left(nChars)
  '--------------------------------------------------------------------------------------
    Local nChars  As Long

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---parse number of chars
    nChars = thinBasic_Parse1Number

    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sLeft(nChars)

    'else
      '---emit some error
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Right(  _
                        ByVal pClass As Long _
                        ) As String
  ' Syntax: 
  '   cString.Right(nChars)
  '--------------------------------------------------------------------------------------
    Local nChars  As Long

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---parse number of chars
    nChars = thinBasic_Parse1Number

    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sRight(nChars)

    'else
      '---emit some error
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Mid(  _
                        ByVal pClass As Long _
                        ) As String
  ' Syntax: 
  '   cString.Mid(Start, Len)
  '--------------------------------------------------------------------------------------
    Local eStart  As Ext
    Local eLen    As Ext

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---parse number of chars
    thinBasic_Parse2Numbers(eStart, eLen)

    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sMid(eStart, eLen)

    'else
      '---emit some error
    End If

  End Function

  '--------------------------------------------------------------------------------------
  Function CString_Len(  _
                        ByVal pClass As Long _
                        ) As Ext
  ' Syntax: 
  '   cString.Len
  '--------------------------------------------------------------------------------------

    '---Remember: programmer is responsible of syntax so wrapper method MUST parse syntax
    
    '---If all went fine ...
    If thinBasic_ErrorFree Then
      'Use the magic REDIM ... AT and the trick is done
      ReDim iClass(1& To 1&) As icString At pClass
      
      '---We return previous value, just in case
      Function = iClass(1&).sLen

    'else
      '---emit some error
    End If

  End Function


  '----------------------------------------------------------------------------
  FUNCTION LoadLocalSymbols ALIAS "LoadLocalSymbols" (OPTIONAL BYVAL sPath AS STRING) EXPORT AS LONG
  ' This function is automatically called by thinCore whenever this DLL is loaded.
  ' This function MUST be present in every external DLL you want to use with thinBasic
  ' Use this function to initialize every variable you need and for loading the
  ' new symbol (read Keyword) you have created.
  '----------------------------------------------------------------------------

    LOCAL RetCode   AS LONG
    LOCAL pClass    AS LONG

    pClass = thinBasic_Class_Add("CString", 0)

    '---If class was created
    IF pClass THEN

      RetCode = thinBasic_Class_AddMethod(pClass, "_Create"       , %thinBasic_ReturnNumber , CodePtr(CString_Create        ))
      RetCode = thinBasic_Class_AddMethod(pClass, "_Destroy"      , %thinBasic_ReturnNumber , CodePtr(CString_Free          ))

      RetCode = thinBasic_Class_AddMethod(pClass, "Set"           , %thinBasic_ReturnString , CodePtr(CString_Set           ))
      RetCode = thinBasic_Class_AddMethod(pClass, "Get"           , %thinBasic_ReturnString , CodePtr(CString_Get           ))
      RetCode = thinBasic_Class_AddMethod(pClass, "Left"          , %thinBasic_ReturnString , CodePtr(CString_Left          ))
      RetCode = thinBasic_Class_AddMethod(pClass, "Right"         , %thinBasic_ReturnString , CodePtr(CString_Right         ))
      RetCode = thinBasic_Class_AddMethod(pClass, "Mid"           , %thinBasic_ReturnString , CodePtr(CString_Mid           ))
      RetCode = thinBasic_Class_AddMethod(pClass, "Len"           , %thinBasic_ReturnNumber , CodePtr(CString_Len           ))
      
    END IF


    FUNCTION = 0&
  END FUNCTION

  '----------------------------------------------------------------------------
  FUNCTION UnLoadLocalSymbols ALIAS "UnLoadLocalSymbols" () EXPORT AS LONG
  ' This function is automatically called by thinCore whenever this DLL is unloaded.
  ' This function CAN be present but it is not necessary.
  ' Use this function to perform uninitialize process, if needed.
  '----------------------------------------------------------------------------


    FUNCTION = 0&
  END FUNCTION


  FUNCTION LIBMAIN ALIAS "LibMain" (BYVAL hInstance   AS LONG, _
                                    BYVAL fwdReason   AS LONG, _
                                    BYVAL lpvReserved AS LONG) Export AS LONG
    SELECT CASE fwdReason
      CASE %DLL_PROCESS_ATTACH

        FUNCTION = 1
        EXIT FUNCTION
      CASE %DLL_PROCESS_DETACH

        FUNCTION = 1
        EXIT FUNCTION
      CASE %DLL_THREAD_ATTACH

        FUNCTION = 1
        EXIT FUNCTION
      CASE %DLL_THREAD_DETACH

        FUNCTION = 1
        EXIT FUNCTION
    END SELECT

  END FUNCTION

